/* wacom.h generated by valac 0.56.18-dirty, the Vala compiler, do not modify */

#ifndef __SRC_WACOM_H__
#define __SRC_WACOM_H__

#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <gee.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <switchboard.h>
#include <gio/gio.h>
#include <gmodule.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif
#define GETTEXT_PACKAGE "io.elementary.settings.wacom"
#define LOCALEDIR "/usr/share/locale"

#define WACOM_BACKEND_TYPE_DEVICE (wacom_backend_device_get_type ())
#define WACOM_BACKEND_DEVICE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), WACOM_BACKEND_TYPE_DEVICE, WacomBackendDevice))
#define WACOM_BACKEND_DEVICE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), WACOM_BACKEND_TYPE_DEVICE, WacomBackendDeviceClass))
#define WACOM_BACKEND_IS_DEVICE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), WACOM_BACKEND_TYPE_DEVICE))
#define WACOM_BACKEND_IS_DEVICE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), WACOM_BACKEND_TYPE_DEVICE))
#define WACOM_BACKEND_DEVICE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), WACOM_BACKEND_TYPE_DEVICE, WacomBackendDeviceClass))

typedef struct _WacomBackendDevice WacomBackendDevice;
typedef struct _WacomBackendDeviceClass WacomBackendDeviceClass;
typedef struct _WacomBackendDevicePrivate WacomBackendDevicePrivate;
typedef enum  {
	WACOM_BACKEND_DEVICE_DEVICE_TYPE_MOUSE = 1 << 0,
	WACOM_BACKEND_DEVICE_DEVICE_TYPE_KEYBOARD = 1 << 1,
	WACOM_BACKEND_DEVICE_DEVICE_TYPE_TOUCHPAD = 1 << 2,
	WACOM_BACKEND_DEVICE_DEVICE_TYPE_TABLET = 1 << 3,
	WACOM_BACKEND_DEVICE_DEVICE_TYPE_TOUCHSCREEN = 1 << 4,
	WACOM_BACKEND_DEVICE_DEVICE_TYPE_PAD = 1 << 5
} WacomBackendDeviceDeviceType;

#define WACOM_BACKEND_DEVICE_TYPE_DEVICE_TYPE (wacom_backend_device_device_type_get_type ())

#define WACOM_BACKEND_TYPE_DEVICE_MANAGER (wacom_backend_device_manager_get_type ())
#define WACOM_BACKEND_DEVICE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), WACOM_BACKEND_TYPE_DEVICE_MANAGER, WacomBackendDeviceManager))
#define WACOM_BACKEND_DEVICE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), WACOM_BACKEND_TYPE_DEVICE_MANAGER, WacomBackendDeviceManagerClass))
#define WACOM_BACKEND_IS_DEVICE_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), WACOM_BACKEND_TYPE_DEVICE_MANAGER))
#define WACOM_BACKEND_IS_DEVICE_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), WACOM_BACKEND_TYPE_DEVICE_MANAGER))
#define WACOM_BACKEND_DEVICE_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), WACOM_BACKEND_TYPE_DEVICE_MANAGER, WacomBackendDeviceManagerClass))

typedef struct _WacomBackendDeviceManager WacomBackendDeviceManager;
typedef struct _WacomBackendDeviceManagerClass WacomBackendDeviceManagerClass;
typedef struct _WacomBackendDeviceManagerPrivate WacomBackendDeviceManagerPrivate;

#define WACOM_BACKEND_TYPE_DEVICE_MANAGER_WAYLAND (wacom_backend_device_manager_wayland_get_type ())
#define WACOM_BACKEND_DEVICE_MANAGER_WAYLAND(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), WACOM_BACKEND_TYPE_DEVICE_MANAGER_WAYLAND, WacomBackendDeviceManagerWayland))
#define WACOM_BACKEND_DEVICE_MANAGER_WAYLAND_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), WACOM_BACKEND_TYPE_DEVICE_MANAGER_WAYLAND, WacomBackendDeviceManagerWaylandClass))
#define WACOM_BACKEND_IS_DEVICE_MANAGER_WAYLAND(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), WACOM_BACKEND_TYPE_DEVICE_MANAGER_WAYLAND))
#define WACOM_BACKEND_IS_DEVICE_MANAGER_WAYLAND_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), WACOM_BACKEND_TYPE_DEVICE_MANAGER_WAYLAND))
#define WACOM_BACKEND_DEVICE_MANAGER_WAYLAND_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), WACOM_BACKEND_TYPE_DEVICE_MANAGER_WAYLAND, WacomBackendDeviceManagerWaylandClass))

typedef struct _WacomBackendDeviceManagerWayland WacomBackendDeviceManagerWayland;
typedef struct _WacomBackendDeviceManagerWaylandClass WacomBackendDeviceManagerWaylandClass;
typedef struct _WacomBackendDeviceManagerWaylandPrivate WacomBackendDeviceManagerWaylandPrivate;

#define WACOM_BACKEND_TYPE_DEVICE_MANAGER_X11 (wacom_backend_device_manager_x11_get_type ())
#define WACOM_BACKEND_DEVICE_MANAGER_X11(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), WACOM_BACKEND_TYPE_DEVICE_MANAGER_X11, WacomBackendDeviceManagerX11))
#define WACOM_BACKEND_DEVICE_MANAGER_X11_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), WACOM_BACKEND_TYPE_DEVICE_MANAGER_X11, WacomBackendDeviceManagerX11Class))
#define WACOM_BACKEND_IS_DEVICE_MANAGER_X11(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), WACOM_BACKEND_TYPE_DEVICE_MANAGER_X11))
#define WACOM_BACKEND_IS_DEVICE_MANAGER_X11_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), WACOM_BACKEND_TYPE_DEVICE_MANAGER_X11))
#define WACOM_BACKEND_DEVICE_MANAGER_X11_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), WACOM_BACKEND_TYPE_DEVICE_MANAGER_X11, WacomBackendDeviceManagerX11Class))

typedef struct _WacomBackendDeviceManagerX11 WacomBackendDeviceManagerX11;
typedef struct _WacomBackendDeviceManagerX11Class WacomBackendDeviceManagerX11Class;
typedef struct _WacomBackendDeviceManagerX11Private WacomBackendDeviceManagerX11Private;

#define TYPE_WACOM_EXCEPTION (wacom_exception_get_type ())

#define WACOM_BACKEND_TYPE_WACOM_TOOL (wacom_backend_wacom_tool_get_type ())
#define WACOM_BACKEND_WACOM_TOOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), WACOM_BACKEND_TYPE_WACOM_TOOL, WacomBackendWacomTool))
#define WACOM_BACKEND_WACOM_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), WACOM_BACKEND_TYPE_WACOM_TOOL, WacomBackendWacomToolClass))
#define WACOM_BACKEND_IS_WACOM_TOOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), WACOM_BACKEND_TYPE_WACOM_TOOL))
#define WACOM_BACKEND_IS_WACOM_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), WACOM_BACKEND_TYPE_WACOM_TOOL))
#define WACOM_BACKEND_WACOM_TOOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), WACOM_BACKEND_TYPE_WACOM_TOOL, WacomBackendWacomToolClass))

typedef struct _WacomBackendWacomTool WacomBackendWacomTool;
typedef struct _WacomBackendWacomToolClass WacomBackendWacomToolClass;
typedef struct _WacomBackendWacomToolPrivate WacomBackendWacomToolPrivate;

#define WACOM_BACKEND_TYPE_WACOM_TOOL_MAP (wacom_backend_wacom_tool_map_get_type ())
#define WACOM_BACKEND_WACOM_TOOL_MAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), WACOM_BACKEND_TYPE_WACOM_TOOL_MAP, WacomBackendWacomToolMap))
#define WACOM_BACKEND_WACOM_TOOL_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), WACOM_BACKEND_TYPE_WACOM_TOOL_MAP, WacomBackendWacomToolMapClass))
#define WACOM_BACKEND_IS_WACOM_TOOL_MAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), WACOM_BACKEND_TYPE_WACOM_TOOL_MAP))
#define WACOM_BACKEND_IS_WACOM_TOOL_MAP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), WACOM_BACKEND_TYPE_WACOM_TOOL_MAP))
#define WACOM_BACKEND_WACOM_TOOL_MAP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), WACOM_BACKEND_TYPE_WACOM_TOOL_MAP, WacomBackendWacomToolMapClass))

typedef struct _WacomBackendWacomToolMap WacomBackendWacomToolMap;
typedef struct _WacomBackendWacomToolMapClass WacomBackendWacomToolMapClass;
typedef struct _WacomBackendWacomToolMapPrivate WacomBackendWacomToolMapPrivate;

#define WACOM_TYPE_STYLUS_VIEW (wacom_stylus_view_get_type ())
#define WACOM_STYLUS_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), WACOM_TYPE_STYLUS_VIEW, WacomStylusView))
#define WACOM_STYLUS_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), WACOM_TYPE_STYLUS_VIEW, WacomStylusViewClass))
#define WACOM_IS_STYLUS_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), WACOM_TYPE_STYLUS_VIEW))
#define WACOM_IS_STYLUS_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), WACOM_TYPE_STYLUS_VIEW))
#define WACOM_STYLUS_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), WACOM_TYPE_STYLUS_VIEW, WacomStylusViewClass))

typedef struct _WacomStylusView WacomStylusView;
typedef struct _WacomStylusViewClass WacomStylusViewClass;
typedef struct _WacomStylusViewPrivate WacomStylusViewPrivate;

#define WACOM_TYPE_TABLET_VIEW (wacom_tablet_view_get_type ())
#define WACOM_TABLET_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), WACOM_TYPE_TABLET_VIEW, WacomTabletView))
#define WACOM_TABLET_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), WACOM_TYPE_TABLET_VIEW, WacomTabletViewClass))
#define WACOM_IS_TABLET_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), WACOM_TYPE_TABLET_VIEW))
#define WACOM_IS_TABLET_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), WACOM_TYPE_TABLET_VIEW))
#define WACOM_TABLET_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), WACOM_TYPE_TABLET_VIEW, WacomTabletViewClass))

typedef struct _WacomTabletView WacomTabletView;
typedef struct _WacomTabletViewClass WacomTabletViewClass;
typedef struct _WacomTabletViewPrivate WacomTabletViewPrivate;

#define WACOM_WIDGETS_TYPE_DRAWING_AREA (wacom_widgets_drawing_area_get_type ())
#define WACOM_WIDGETS_DRAWING_AREA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), WACOM_WIDGETS_TYPE_DRAWING_AREA, WacomWidgetsDrawingArea))
#define WACOM_WIDGETS_DRAWING_AREA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), WACOM_WIDGETS_TYPE_DRAWING_AREA, WacomWidgetsDrawingAreaClass))
#define WACOM_WIDGETS_IS_DRAWING_AREA(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), WACOM_WIDGETS_TYPE_DRAWING_AREA))
#define WACOM_WIDGETS_IS_DRAWING_AREA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), WACOM_WIDGETS_TYPE_DRAWING_AREA))
#define WACOM_WIDGETS_DRAWING_AREA_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), WACOM_WIDGETS_TYPE_DRAWING_AREA, WacomWidgetsDrawingAreaClass))

typedef struct _WacomWidgetsDrawingArea WacomWidgetsDrawingArea;
typedef struct _WacomWidgetsDrawingAreaClass WacomWidgetsDrawingAreaClass;
typedef struct _WacomWidgetsDrawingAreaPrivate WacomWidgetsDrawingAreaPrivate;

#define WACOM_TYPE_MAIN_PAGE (wacom_main_page_get_type ())
#define WACOM_MAIN_PAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), WACOM_TYPE_MAIN_PAGE, WacomMainPage))
#define WACOM_MAIN_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), WACOM_TYPE_MAIN_PAGE, WacomMainPageClass))
#define WACOM_IS_MAIN_PAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), WACOM_TYPE_MAIN_PAGE))
#define WACOM_IS_MAIN_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), WACOM_TYPE_MAIN_PAGE))
#define WACOM_MAIN_PAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), WACOM_TYPE_MAIN_PAGE, WacomMainPageClass))

typedef struct _WacomMainPage WacomMainPage;
typedef struct _WacomMainPageClass WacomMainPageClass;
typedef struct _WacomMainPagePrivate WacomMainPagePrivate;

#define WACOM_TYPE_PLUG (wacom_plug_get_type ())
#define WACOM_PLUG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), WACOM_TYPE_PLUG, WacomPlug))
#define WACOM_PLUG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), WACOM_TYPE_PLUG, WacomPlugClass))
#define WACOM_IS_PLUG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), WACOM_TYPE_PLUG))
#define WACOM_IS_PLUG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), WACOM_TYPE_PLUG))
#define WACOM_PLUG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), WACOM_TYPE_PLUG, WacomPlugClass))

typedef struct _WacomPlug WacomPlug;
typedef struct _WacomPlugClass WacomPlugClass;
typedef struct _WacomPlugPrivate WacomPlugPrivate;

struct _WacomBackendDevice {
	GObject parent_instance;
	WacomBackendDevicePrivate * priv;
};

struct _WacomBackendDeviceClass {
	GObjectClass parent_class;
};

struct _WacomBackendDeviceManager {
	GObject parent_instance;
	WacomBackendDeviceManagerPrivate * priv;
};

struct _WacomBackendDeviceManagerClass {
	GObjectClass parent_class;
	GeeArrayList* (*list_devices) (WacomBackendDeviceManager* self, WacomBackendDeviceDeviceType type);
	WacomBackendDevice* (*lookup_gdk_device) (WacomBackendDeviceManager* self, GdkDevice* device);
};

struct _WacomBackendDeviceManagerWayland {
	WacomBackendDeviceManager parent_instance;
	WacomBackendDeviceManagerWaylandPrivate * priv;
};

struct _WacomBackendDeviceManagerWaylandClass {
	WacomBackendDeviceManagerClass parent_class;
};

struct _WacomBackendDeviceManagerX11 {
	WacomBackendDeviceManager parent_instance;
	WacomBackendDeviceManagerX11Private * priv;
};

struct _WacomBackendDeviceManagerX11Class {
	WacomBackendDeviceManagerClass parent_class;
};

typedef enum  {
	WACOM_EXCEPTION_LIBWACOM_ERROR
} WacomException;
#define WACOM_EXCEPTION wacom_exception_quark ()

struct _WacomBackendWacomTool {
	GObject parent_instance;
	WacomBackendWacomToolPrivate * priv;
};

struct _WacomBackendWacomToolClass {
	GObjectClass parent_class;
};

struct _WacomBackendWacomToolMap {
	GObject parent_instance;
	WacomBackendWacomToolMapPrivate * priv;
};

struct _WacomBackendWacomToolMapClass {
	GObjectClass parent_class;
};

struct _WacomStylusView {
	GtkBox parent_instance;
	WacomStylusViewPrivate * priv;
};

struct _WacomStylusViewClass {
	GtkBoxClass parent_class;
};

struct _WacomTabletView {
	GtkGrid parent_instance;
	WacomTabletViewPrivate * priv;
};

struct _WacomTabletViewClass {
	GtkGridClass parent_class;
};

struct _WacomWidgetsDrawingArea {
	GtkDrawingArea parent_instance;
	WacomWidgetsDrawingAreaPrivate * priv;
};

struct _WacomWidgetsDrawingAreaClass {
	GtkDrawingAreaClass parent_class;
};

struct _WacomMainPage {
	SwitchboardSettingsPage parent_instance;
	WacomMainPagePrivate * priv;
};

struct _WacomMainPageClass {
	SwitchboardSettingsPageClass parent_class;
};

struct _WacomPlug {
	SwitchboardPlug parent_instance;
	WacomPlugPrivate * priv;
};

struct _WacomPlugClass {
	SwitchboardPlugClass parent_class;
};

VALA_EXTERN GType wacom_backend_device_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (WacomBackendDevice, g_object_unref)
VALA_EXTERN GType wacom_backend_device_device_type_get_type (void) G_GNUC_CONST ;
VALA_EXTERN WacomBackendDevice* wacom_backend_device_new (void);
VALA_EXTERN WacomBackendDevice* wacom_backend_device_construct (GType object_type);
VALA_EXTERN const gchar* wacom_backend_device_get_device_file (WacomBackendDevice* self);
VALA_EXTERN void wacom_backend_device_set_device_file (WacomBackendDevice* self,
                                           const gchar* value);
VALA_EXTERN const gchar* wacom_backend_device_get_vendor_id (WacomBackendDevice* self);
VALA_EXTERN void wacom_backend_device_set_vendor_id (WacomBackendDevice* self,
                                         const gchar* value);
VALA_EXTERN const gchar* wacom_backend_device_get_product_id (WacomBackendDevice* self);
VALA_EXTERN void wacom_backend_device_set_product_id (WacomBackendDevice* self,
                                          const gchar* value);
VALA_EXTERN WacomBackendDeviceDeviceType wacom_backend_device_get_dev_type (WacomBackendDevice* self);
VALA_EXTERN void wacom_backend_device_set_dev_type (WacomBackendDevice* self,
                                        WacomBackendDeviceDeviceType value);
VALA_EXTERN GType wacom_backend_device_manager_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (WacomBackendDeviceManager, g_object_unref)
VALA_EXTERN WacomBackendDeviceManager* wacom_backend_device_manager_get_default (void);
VALA_EXTERN GeeArrayList* wacom_backend_device_manager_list_devices (WacomBackendDeviceManager* self,
                                                         WacomBackendDeviceDeviceType type);
VALA_EXTERN WacomBackendDevice* wacom_backend_device_manager_lookup_gdk_device (WacomBackendDeviceManager* self,
                                                                    GdkDevice* device);
VALA_EXTERN WacomBackendDeviceManager* wacom_backend_device_manager_construct (GType object_type);
VALA_EXTERN GType wacom_backend_device_manager_wayland_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (WacomBackendDeviceManagerWayland, g_object_unref)
VALA_EXTERN WacomBackendDeviceManagerWayland* wacom_backend_device_manager_wayland_new (void);
VALA_EXTERN WacomBackendDeviceManagerWayland* wacom_backend_device_manager_wayland_construct (GType object_type);
VALA_EXTERN GType wacom_backend_device_manager_x11_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (WacomBackendDeviceManagerX11, g_object_unref)
VALA_EXTERN WacomBackendDeviceManagerX11* wacom_backend_device_manager_x11_new (void);
VALA_EXTERN WacomBackendDeviceManagerX11* wacom_backend_device_manager_x11_construct (GType object_type);
VALA_EXTERN GdkDeviceToolType wacom_backend_device_manager_x11_get_tool_type (GdkDevice* device);
VALA_EXTERN GQuark wacom_exception_quark (void);
VALA_EXTERN GType wacom_exception_get_type (void) G_GNUC_CONST ;
VALA_EXTERN GType wacom_backend_wacom_tool_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (WacomBackendWacomTool, g_object_unref)
VALA_EXTERN WacomBackendWacomTool* wacom_backend_wacom_tool_new (guint64 serial,
                                                     guint64 id,
                                                     const gchar* settings_path);
VALA_EXTERN WacomBackendWacomTool* wacom_backend_wacom_tool_construct (GType object_type,
                                                           guint64 serial,
                                                           guint64 id,
                                                           const gchar* settings_path);
VALA_EXTERN guint64 wacom_backend_wacom_tool_get_id (WacomBackendWacomTool* self);
VALA_EXTERN guint64 wacom_backend_wacom_tool_get_serial (WacomBackendWacomTool* self);
VALA_EXTERN const gchar* wacom_backend_wacom_tool_get_settings_path (WacomBackendWacomTool* self);
VALA_EXTERN GType wacom_backend_wacom_tool_map_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (WacomBackendWacomToolMap, g_object_unref)
VALA_EXTERN WacomBackendWacomToolMap* wacom_backend_wacom_tool_map_get_default (void);
VALA_EXTERN WacomBackendWacomToolMap* wacom_backend_wacom_tool_map_new (void);
VALA_EXTERN WacomBackendWacomToolMap* wacom_backend_wacom_tool_map_construct (GType object_type);
VALA_EXTERN void wacom_backend_wacom_tool_map_add_relation (WacomBackendWacomToolMap* self,
                                                const gchar* device_key,
                                                WacomBackendWacomTool* tool);
VALA_EXTERN WacomBackendWacomTool* wacom_backend_wacom_tool_map_lookup_tool (WacomBackendWacomToolMap* self,
                                                                 guint64 serial,
                                                                 const gchar* device_key);
VALA_EXTERN GeeArrayList* wacom_backend_wacom_tool_map_list_tools (WacomBackendWacomToolMap* self,
                                                       WacomBackendDevice* device,
                                                       GError** error);
VALA_EXTERN GType wacom_stylus_view_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (WacomStylusView, g_object_unref)
VALA_EXTERN gboolean wacom_stylus_view_is_stylus_supported (WacomStylusView* self,
                                                WacomBackendWacomTool* wacom_tool);
VALA_EXTERN void wacom_stylus_view_set_device (WacomStylusView* self,
                                   WacomBackendWacomTool* wacom_tool);
VALA_EXTERN WacomStylusView* wacom_stylus_view_new (void);
VALA_EXTERN WacomStylusView* wacom_stylus_view_construct (GType object_type);
VALA_EXTERN GType wacom_tablet_view_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (WacomTabletView, g_object_unref)
VALA_EXTERN WacomTabletView* wacom_tablet_view_new (void);
VALA_EXTERN WacomTabletView* wacom_tablet_view_construct (GType object_type);
VALA_EXTERN const gchar* wacom_tablet_view_get_device_key (WacomTabletView* self);
VALA_EXTERN void wacom_tablet_view_set_device_key (WacomTabletView* self,
                                       const gchar* value);
VALA_EXTERN GType wacom_widgets_drawing_area_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (WacomWidgetsDrawingArea, g_object_unref)
VALA_EXTERN void wacom_widgets_drawing_area_clear (WacomWidgetsDrawingArea* self);
VALA_EXTERN WacomWidgetsDrawingArea* wacom_widgets_drawing_area_new (void);
VALA_EXTERN WacomWidgetsDrawingArea* wacom_widgets_drawing_area_construct (GType object_type);
VALA_EXTERN GType wacom_main_page_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (WacomMainPage, g_object_unref)
VALA_EXTERN WacomMainPage* wacom_main_page_new (void);
VALA_EXTERN WacomMainPage* wacom_main_page_construct (GType object_type);
VALA_EXTERN GType wacom_plug_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (WacomPlug, g_object_unref)
VALA_EXTERN WacomPlug* wacom_plug_new (void);
VALA_EXTERN WacomPlug* wacom_plug_construct (GType object_type);
VALA_EXTERN SwitchboardPlug* get_plug (GModule* module);

G_END_DECLS

#endif
